from abc import abstractmethod
import ast
from typing import List
from turms.config import GeneratorConfig
from graphql.utilities.build_client_schema import GraphQLSchema


class Plugin:
    """Base Plugin Class

    Raises:
        NotImplementedError: [description]
    """

    def generate_imports(
        self, config: GeneratorConfig, client_schema: GraphQLSchema
    ) -> List[ast.AST]:
        return []

    @abstractmethod
    def generate_body(
        self, config: GeneratorConfig, client_schema: GraphQLSchema
    ) -> List[ast.AST]:
        raise NotImplementedError("Plugin must overrwrite this")



FRAGMENT_DOCUMENT_MAP = {}  # TODO: THAT NEEDS TO DIE
ENUM_CLASS_MAP = {}  # TODO: THAT NEEDS TO DIE
INPUTTYPE_CLASS_MAP = {}
FRAGMENT_CLASS_MAP = {}


SCALAR_DEFAULTS = {
    "ID": "str",
    "String": "str",
    "Int": "int",
    "Boolean": "bool",
    "GenericScalar": "Dict",
    "DateTime": "str",
    "Float": "float",
}



import copy
import os
import sys
from importlib import import_module
from importlib.util import find_spec as importlib_find


def import_class(module_path, class_name):
    module = import_module(module_path)
    return getattr(module, class_name)


def import_string(dotted_path):
    """
    Import a dotted module path and return the attribute/class designated by the
    last name in the path. Raise ImportError if the import failed. Simliar to
    djangos import_string, but without the cache.
    """

    try:
        module_path, class_name = dotted_path.rsplit(".", 1)
    except ValueError as err:
        raise ImportError(f"{dotted_path} doesn't look like a module path") from err

    try:
        return import_class(module_path, class_name)
    except AttributeError as err:
        raise ImportError(
            f"{module_path} does not define a {class_name} attribute/class"
        ) from err
